BACKEND_OPTIONS = ["swaybg", "swww", "feh", "wallutils"]
FILL_OPTIONS = ["fill", "stretch", "fit", "center", "tile"]
SORT_OPTIONS = ["name", "namerev", "date", "daterev"]
SORT_DISPLAYS = {
                "name": "Sort by name ↓",
                "namerev": "Sort by name ↑",
                "date": "Sort by date ↓",
                "daterev": "Sort by date ↑"}
